texture RenderScene;

sampler2D sRenderScene =
sampler_state
{
    Texture = <RenderScene>;
    AddressU = Clamp;
    AddressV = Clamp;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = None;
};

float4x4 matWVP : WorldViewProjection;

struct vertexInput {
	float3 position			: POSITION;
	float4 normal			: NORMAL;
	float4 texCoord0		: TEXCOORD0;
	float4 diffuse			: COLOR0;
};

struct vertexOutputR {
	float4 position			: POSITION;
	float4 texCoord0		: TEXCOORD0;
	float4 texCoord1		: TEXCOORD1;
	float4 texCoord2		: TEXCOORD2;
};

struct vertexOutputT {
	float4 position			: POSITION;
	float4 texCoord0		: TEXCOORD0;
	float4 diffuse			: COLOR0;
};

vertexOutputR VS_Refract(vertexInput IN) 
{
	vertexOutputR OUT;
	OUT.position = mul(float4(IN.position.xyz, 1.0), matWVP);
	OUT.position /= abs(OUT.position.w);
	OUT.texCoord0 = IN.texCoord0;
	OUT.texCoord1 = mul(float4(IN.normal.xyz, 1.0), matWVP);
	OUT.texCoord1 /= abs(OUT.texCoord1.w);
	OUT.texCoord2 = float4((OUT.texCoord1.x / OUT.texCoord1.w) / 2.0 + 0.5, (-OUT.texCoord1.y / OUT.texCoord1.w) / 2.0 + 0.5, 0.0, 0.0);
	OUT.texCoord1 = float4((OUT.position.x / OUT.position.w) / 2.0 + 0.5, (-OUT.position.y / OUT.position.w) / 2.0 + 0.5, 0.0, 0.0);
	return OUT;
}

float4 PS_Refract(vertexOutputR IN) : COLOR
{
	return float4(tex2D(sRenderScene, lerp(IN.texCoord2, IN.texCoord1, 2.0 - min(length(IN.texCoord0.xy), 1.0))).rgb, ceil(1.0 - length(IN.texCoord0.xy)));
}

vertexOutputT VS_Tracer(vertexInput IN) 
{
	vertexOutputT OUT;
	OUT.position = mul(float4(IN.position.xyz, 1.0), matWVP);
	OUT.texCoord0 = IN.texCoord0;
	OUT.diffuse = IN.diffuse;
	return OUT;
}

float4 PS_Tracer(vertexOutputT IN) : COLOR
{
	return float4(IN.diffuse.rgb * IN.diffuse.a * 255.0 / 16.0 * min(max(1.0 - length(IN.texCoord0.xy), 0.0) / IN.texCoord0.z, 1.0) + IN.texCoord0.www, 1.0);
}

technique PostProcess
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS_Refract();
        PixelShader = compile ps_2_0 PS_Refract();
    }
    pass p1
    {
        VertexShader = compile vs_2_0 VS_Tracer();
        PixelShader = compile ps_2_0 PS_Tracer();
    }
}
